//------------------------------------------------------------------------------------------------
//
//   Greg's Mod Base - Generic Mod
//
//------------------------------------------------------------------------------------------------

package gcewing.sg;

import java.io.*;
import java.lang.reflect.*;
import java.net.*;
import java.util.*;
import java.util.jar.*;

import net.minecraft.block.*;
import net.minecraft.entity.player.*;
import net.minecraft.item.*;
import net.minecraft.world.*;

import net.minecraftforge.common.*;
import net.minecraftforge.client.*;
import cpw.mods.fml.common.*;
import cpw.mods.fml.common.event.*;
import cpw.mods.fml.common.network.*;
import cpw.mods.fml.common.registry.*;

public class BaseMod {

	public static BaseConfiguration config;
	public static String modPackage;
	public static String resourceDir; // path to resources directory with leading and trailing slashes
	public static String textureFile; // path to default texture file with leading slash
	public static URL resourceURL; // URL to the resources directory
	public static BaseMod base;
	public static Proxy proxy;
	public static String channel;

	static File cfgFile;

	static String resourcePath(String fileName) {
		return resourceDir + fileName;
	}

	public BaseMod(String pkg) {
		base = this;
		modPackage = pkg;
		String resourceRelDir = modPackage.replace(".", "/") + "/resources/";
		resourceDir = "/" + resourceRelDir;
		textureFile = resourceDir + "textures.png";
		resourceURL = getClass().getClassLoader().getResource(resourceRelDir);
		System.out.printf("BaseMod: resourceURL = %s\n", resourceURL);
	}

	//@Mod.PreInit
	public void preInit(FMLPreInitializationEvent e) {
		cfgFile = e.getSuggestedConfigurationFile();
		if (e.getSide().isClient())
			proxy = new ProxyClient();
		else
			proxy = new Proxy();
		System.out.printf("BaseMod: Registering packet handlers for channel '%s'\n", channel);
	}
	
	//@Mod.Init
	public void init(FMLInitializationEvent e) {
		MinecraftForge.EVENT_BUS.register(this);
		NetworkRegistry.instance().registerGuiHandler(this, proxy);
		loadConfig();
		registerBlocks();
		registerItems();
		registerRecipes();
		registerTileEntities();
		proxy.init(this);
	}
	
	//@Mod.PostInit
	public void postInit(FMLPostInitializationEvent e) {
		saveConfig();
	}

	void loadConfig() {
		config = new BaseConfiguration(cfgFile);
	}

	void saveConfig() {
		if (config.extended)
			config.save();
	}

	String qualifiedName(String name) {
		return "gcewing." + modPackage + "." + name;
	}

	BaseItem newItem(String name, int icon, String title) {
		return newItem(name, icon, BaseItem.class, title);
	}

	<ITEM extends BaseItem> ITEM newItem(String name, int icon, Class<ITEM> cls, String title) {
		try {
			int id = config.getItem(name, 1).getInt();
			Constructor<ITEM> ctor = cls.getConstructor(int.class, String.class);
			ITEM item = ctor.newInstance(id, textureFile);
			item.setItemName(qualifiedName(name));
			item.setIconIndex(icon);
			LanguageRegistry.addName(item, title);
			return item;
		}
		catch (Exception e) {
			throw new RuntimeException(e.toString());
		}
	}
	
	Block newBlock(String name, String title) {
		return newBlock(name, Block.class, title);
	}
	
	<BLOCK extends Block> BLOCK newBlock(String name, Class<BLOCK> cls, String title) {
		return newBlock(name, cls, ItemBlock.class, title);
	}
	
	<BLOCK extends Block> BLOCK newBlock(String name, Class<BLOCK> cls, Class itemClass, String title) {
		try {
			int id = config.getBlock(name, 1).getInt();
			Constructor<BLOCK> ctor = cls.getConstructor(int.class);
			BLOCK block = ctor.newInstance(id);
			block.setBlockName(qualifiedName(name));
			GameRegistry.registerBlock(block, itemClass);
			LanguageRegistry.addName(block, title);
			return block;
		}
		catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	void newRecipe(Item product, int qty, Object... params) {
		GameRegistry.addRecipe(new ItemStack(product, qty), params);
	}

	void newRecipe(Block product, int qty, Object... params) {
		GameRegistry.addRecipe(new ItemStack(product, qty), params);
	}

	void newRecipe(ItemStack product, Object... params) {
		GameRegistry.addRecipe(product, params);
	}

	void newShapelessRecipe(Item product, int qty, Object... params) {
		GameRegistry.addShapelessRecipe(new ItemStack(product, qty), params);
	}

	void newSmeltingRecipe(Item product, int qty, Item input) {
		GameRegistry.addSmelting(input.shiftedIndex, new ItemStack(product, qty), 0);
	}
	
	void newSmeltingRecipe(Item product, int qty, Block input) {
		GameRegistry.addSmelting(input.blockID, new ItemStack(product, qty), 0);
	}
	
	void registerBlocks() {}
	void registerItems() {}
	void registerRecipes() {}
	void registerTileEntities() {}
	
	public Set<String> listResources(String subdir) {
		try {
			Set<String>result = new HashSet<String>();
			String protocol = resourceURL.getProtocol();
			if (protocol.equals("jar")) {
				String resPath = resourceURL.getPath();
				int pling = resPath.indexOf("!");
				URL jarURL = new URL(resPath.substring(0, pling));
				String resDirInJar = resPath.substring(pling + 2);
				String prefix = resDirInJar + subdir + "/";
				//System.out.printf("BaseMod.listResources: looking for names starting with %s\n", prefix);
				JarFile jar = new JarFile(new File(jarURL.toURI()));
				Enumeration<JarEntry> entries = jar.entries();
				while (entries.hasMoreElements()) {
					String name = entries.nextElement().getName();
					if (name.startsWith(prefix) && !name.endsWith("/") && !name.contains("/.")) {
						//System.out.printf("BaseMod.listResources: name = %s\n", name);
						result.add(name.substring(prefix.length()));
					}
				}
			}
			else
				throw new RuntimeException("Resource URL protocol " + protocol + " not supported");
			return result;
		}
		catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
	
	static void openGui(EntityPlayer player, Enum id, World world, int x, int y, int z) {
		openGui(player, id.ordinal(), world, x, y, z);
	}

	static void openGui(EntityPlayer player, int id, World world, int x, int y, int z) {
		player.openGui(base, id, world, x, y, z);
	}
	
}
